searchData={"items":[{"type":"module","title":"Graphmath.Mat33","doc":"This is the 3D mathematics.\n\nThis submodule handles 3x3 matrices using tuples of floats.","ref":"Graphmath.Mat33.html"},{"type":"function","title":"Graphmath.Mat33.add/2","doc":"`add(a,b)` adds one `mat33` to another `mat33`.\n\n`a` is the first `mat33`.\n\n`b` is the second `mat33`.\n\nThis returns a `mat33` which is the element-wise sum of `a` and `b`.","ref":"Graphmath.Mat33.html#add/2"},{"type":"function","title":"Graphmath.Mat33.apply/2","doc":"`apply( a, v )` transforms a `vec3` by a `mat33`.\n\n`a` is the `mat33` to transform by.\n\n`v` is the `vec3` to be transformed.\n\nThis returns a `vec3` representing **A****v**.\n\nThis is the \"full\" application of a matrix, and uses all elements.","ref":"Graphmath.Mat33.html#apply/2"},{"type":"function","title":"Graphmath.Mat33.apply_left/2","doc":"`apply_left( v, a )` transforms a `vec3` by a `mat33`, applied on the left.\n\n`a` is the `mat33` to transform by.\n\n`v` is the `vec3` to be transformed.\n\nThis returns a `vec3` representing **v****A**.\n\nThis is the \"full\" application of a matrix, and uses all elements.","ref":"Graphmath.Mat33.html#apply_left/2"},{"type":"function","title":"Graphmath.Mat33.apply_left_transpose/2","doc":"`apply_left_transpose( v, a )` transforms a `vec3` by a transposed `mat33`, applied on the left.\n\n`a` is the `mat33` to transform by.\n\n`v` is the `vec3` to be transformed.\n\nThis returns a `vec3` representing **v****A** T .\n\nThis is the \"full\" application of a matrix, and uses all elements.","ref":"Graphmath.Mat33.html#apply_left_transpose/2"},{"type":"function","title":"Graphmath.Mat33.apply_transpose/2","doc":"`apply_transpose( a, v )` transforms a `vec3` by a a transposed `mat33`.\n\n`a` is the `mat33` to transform by.\n\n`v` is the `vec3` to be transformed.\n\nThis returns a `vec3` representing **A** T **v**.\n\nThis is the \"full\" application of a matrix, and uses all elements.","ref":"Graphmath.Mat33.html#apply_transpose/2"},{"type":"function","title":"Graphmath.Mat33.at/3","doc":"`at( a, i, j)` selects an element of a `mat33`.\n\n`a` is the `mat33` to index.\n\n`i` is the row integer index [0,2].\n\n`j` is the column integer index [0,2].\n\nThis returns a float from the matrix at row `i` and column `j`.","ref":"Graphmath.Mat33.html#at/3"},{"type":"function","title":"Graphmath.Mat33.column0/1","doc":"`column0( a )` selects the first column of a `mat33`.\n\n`a` is the `mat33` to take the first column of.\n\nThis returns a `vec3` representing the first column of `a`.","ref":"Graphmath.Mat33.html#column0/1"},{"type":"function","title":"Graphmath.Mat33.column1/1","doc":"`column1( a )` selects the second column of a `mat33`.\n\n`a` is the `mat33` to take the second column of.\n\nThis returns a `vec3` representing the second column of `a`.","ref":"Graphmath.Mat33.html#column1/1"},{"type":"function","title":"Graphmath.Mat33.column2/1","doc":"`column2( a )` selects the third column of a `mat33`.\n\n`a` is the `mat33` to take the third column of.\n\nThis returns a `vec3` representing the third column of `a`.","ref":"Graphmath.Mat33.html#column2/1"},{"type":"function","title":"Graphmath.Mat33.diag/1","doc":"`diag( a )` selects the diagonal of a `mat33`.\n\n`a` is the `mat33` to take the diagonal of.\n\nThis returns a `vec3` representing the diagonal of `a`.","ref":"Graphmath.Mat33.html#diag/1"},{"type":"function","title":"Graphmath.Mat33.identity/0","doc":"`identity()` creates an identity `mat33`.\n\nThis returns an identity `mat33`.","ref":"Graphmath.Mat33.html#identity/0"},{"type":"function","title":"Graphmath.Mat33.inverse/1","doc":"`inverse(a)` calculates the inverse matrix\n\n`a` is a `mat33` to be inverted\n\nReturs a `mat33` representing `a` -1 \n\nRaises an error when you try to calculate inverse of a matrix whose determinant is `zero`","ref":"Graphmath.Mat33.html#inverse/1"},{"type":"function","title":"Graphmath.Mat33.make_rotate/1","doc":"`make_rotate( theta )` creates a mat33 that rotates a vec2 by `theta` radians about the +Z axis.\n\n`theta` is the float of the number of radians of rotation the matrix will provide.\n\nThis returns a `mat33` which rotates by `theta` radians about the +Z axis.","ref":"Graphmath.Mat33.html#make_rotate/1"},{"type":"function","title":"Graphmath.Mat33.make_scale/1","doc":"`make_scale( k )` creates a `mat33` that uniformly scales.\n\n`k` is the float value to scale by.\n\nThis returns a `mat33` whose diagonal is all `k`s.","ref":"Graphmath.Mat33.html#make_scale/1"},{"type":"function","title":"Graphmath.Mat33.make_scale/3","doc":"`make_scale( sx, sy, sz )` creates a `mat33` that scales each axis independently.\n\n`sx` is a float for scaling the x-axis.\n\n`sy` is a float for scaling the y-axis.\n\n`sz` is a float for scaling the z-axis.\n\nThis returns a `mat33` whose diagonal is `{ sx, sy, sz }`.\n\nNote that, when used with `vec2`s via the *transform* methods, `sz` will have no effect.","ref":"Graphmath.Mat33.html#make_scale/3"},{"type":"function","title":"Graphmath.Mat33.make_translate/2","doc":"`make_translate( tx, ty )` creates a mat33 that translates a vec2 by (tx, ty).\n\n`tx` is a float for translating along the x-axis.\n\n`ty` is a float for translating along the y-axis.\n\nThis returns a `mat33` which translates by a `vec2` `{ tx, ty }`.","ref":"Graphmath.Mat33.html#make_translate/2"},{"type":"function","title":"Graphmath.Mat33.multiply/2","doc":"`multiply( a, b )` multiply two matrices a and b together.\n\n`a` is the `mat33` multiplicand.\n\n`b` is the `mat33` multiplier.\n\nThis returns the `mat33` product of the `a` and `b`.","ref":"Graphmath.Mat33.html#multiply/2"},{"type":"function","title":"Graphmath.Mat33.multiply_transpose/2","doc":"`multiply_transpose( a, b )` multiply two matrices a and b T  together.\n\n`a` is the `mat33` multiplicand.\n\n`b` is the `mat33` multiplier.\n\nThis returns the `mat33` product of the `a` and `b` T .","ref":"Graphmath.Mat33.html#multiply_transpose/2"},{"type":"function","title":"Graphmath.Mat33.round/2","doc":"`round( a, sigfigs )` rounds every element of a `mat33` to some number of decimal places.\n\n`a` is the `mat33` to round.\n\n`sigfigs` is an integer on [0,15] of the number of decimal places to round to.\n\nThis returns a `mat33` which is the result of rounding `a`.","ref":"Graphmath.Mat33.html#round/2"},{"type":"function","title":"Graphmath.Mat33.row0/1","doc":"`row0( a )` selects the first row of a `mat33`.\n\n`a` is the `mat33` to take the first row of.\n\nThis returns a `vec3` representing the first row of `a`.","ref":"Graphmath.Mat33.html#row0/1"},{"type":"function","title":"Graphmath.Mat33.row1/1","doc":"`row1( a )` selects the second row of a `mat33`.\n\n`a` is the `mat33` to take the second row of.\n\nThis returns a `vec3` representing the second row of `a`.","ref":"Graphmath.Mat33.html#row1/1"},{"type":"function","title":"Graphmath.Mat33.row2/1","doc":"`row2( a )` selects the third row of a `mat33`.\n\n`a` is the `mat33` to take the third row of.\n\nThis returns a `vec3` representing the third row of `a`.","ref":"Graphmath.Mat33.html#row2/1"},{"type":"function","title":"Graphmath.Mat33.scale/2","doc":"`scale( a, k )` scales every element in a `mat33` by a coefficient k.\n\n`a` is the `mat33` to scale.\n\n`k` is the float to scale by.\n\nThis returns a `mat33` `a` scaled element-wise by `k`.","ref":"Graphmath.Mat33.html#scale/2"},{"type":"function","title":"Graphmath.Mat33.subtract/2","doc":"`subtract(a,b)` subtracts one `mat33` from another `mat33`.\n\n`a` is the minuend.\n\n`b` is the subtraherd.\n\nThis returns a `mat33` formed by the element-wise subtraction of `b` from `a`.","ref":"Graphmath.Mat33.html#subtract/2"},{"type":"function","title":"Graphmath.Mat33.transform_point/2","doc":"`transform_point( a, v )` transforms a `vec2` point by a `mat33`.\n\n`a` is a `mat33` used to transform the point.\n\n`v` is a `vec2` to be transformed.\n\nThis returns a `vec2` representing the application of `a` to `v`.\n\nThe point `a` is internally treated as having a third coordinate equal to 1.0.\n\nNote that transforming a point will work for all transforms.","ref":"Graphmath.Mat33.html#transform_point/2"},{"type":"function","title":"Graphmath.Mat33.transform_vector/2","doc":"`transform_vector( a, v )` transforms a `vec2` vector by a `mat33`.\n\n`a` is a `mat33` used to transform the point.\n\n`v` is a `vec2` to be transformed.\n\nThis returns a `vec2` representing the application of `a` to `v`.\n\nThe point `a` is internally treated as having a third coordinate equal to 0.0.\n\nNote that transforming a vector will work for only rotations, scales, and shears.","ref":"Graphmath.Mat33.html#transform_vector/2"},{"type":"function","title":"Graphmath.Mat33.zero/0","doc":"`zero()` creates a zeroed `mat33`.\n\nThis returns a zeroed `mat33`.","ref":"Graphmath.Mat33.html#zero/0"},{"type":"type","title":"Graphmath.Mat33.mat33/0","doc":"","ref":"Graphmath.Mat33.html#t:mat33/0"},{"type":"type","title":"Graphmath.Mat33.vec2/0","doc":"","ref":"Graphmath.Mat33.html#t:vec2/0"},{"type":"type","title":"Graphmath.Mat33.vec3/0","doc":"","ref":"Graphmath.Mat33.html#t:vec3/0"},{"type":"module","title":"Graphmath.Mat44","doc":"This is the 3D mathematics.\n\nThis submodule handles 4x4 matrices using tuples of floats.","ref":"Graphmath.Mat44.html"},{"type":"function","title":"Graphmath.Mat44.add/2","doc":"`add(a,b)` adds one `mat44` to another `mat44`.\n\n`a` is the first `mat44`.\n\n`b` is the second `mat44`.\n\nThis returns a `mat44` which is the element-wise sum of `a` and `b`.","ref":"Graphmath.Mat44.html#add/2"},{"type":"function","title":"Graphmath.Mat44.apply/2","doc":"`apply( a, v )` transforms a `vec4` by a `mat44`.\n\n`a` is the `mat44` to transform by.\n\n`v` is the `vec4` to be transformed.\n\nThis returns a `vec4` representing **A****v**.\n\nThis is the \"full\" application of a matrix, and uses all elements.","ref":"Graphmath.Mat44.html#apply/2"},{"type":"function","title":"Graphmath.Mat44.apply_left/2","doc":"`apply_left( v, a )` transforms a `vec4` by a `mat44`, applied on the left.\n\n`a` is the `mat44` to transform by.\n\n`v` is the `vec4` to be transformed.\n\nThis returns a `vec4` representing **v****A**.\n\nThis is the \"full\" application of a matrix, and uses all elements.","ref":"Graphmath.Mat44.html#apply_left/2"},{"type":"function","title":"Graphmath.Mat44.apply_left_transpose/2","doc":"`apply_left_transpose( v, a )` transforms a `vec3` by a transposed `mat33`, applied on the left.\n\n`a` is the `mat44` to transform by.\n\n`v` is the `vec4` to be transformed.\n\nThis returns a `vec4` representing **v****A** T .\n\nThis is the \"full\" application of a matrix, and uses all elements.","ref":"Graphmath.Mat44.html#apply_left_transpose/2"},{"type":"function","title":"Graphmath.Mat44.apply_transpose/2","doc":"`apply_transpose( a, v )` transforms a `vec4` by a a transposed `mat44`.\n\n`a` is the `mat44` to transform by.\n\n`v` is the `vec4` to be transformed.\n\nThis returns a `vec4` representing **A** T **v**.\n\nThis is the \"full\" application of a matrix, and uses all elements.","ref":"Graphmath.Mat44.html#apply_transpose/2"},{"type":"function","title":"Graphmath.Mat44.at/3","doc":"`at( a, i, j)` selects an element of a `mat44`.\n\n`a` is the `mat44` to index.\n\n`i` is the row integer index [0,3].\n\n`j` is the column integer index [0,3].\n\nThis returns a float from the matrix at row `i` and column `j`.","ref":"Graphmath.Mat44.html#at/3"},{"type":"function","title":"Graphmath.Mat44.column0/1","doc":"`column0( a )` selects the first column of a `mat44`.\n\n`a` is the `mat44` to take the first column of.\n\nThis returns a `vec4` representing the first column of `a`.","ref":"Graphmath.Mat44.html#column0/1"},{"type":"function","title":"Graphmath.Mat44.column1/1","doc":"`column1( a )` selects the second column of a `mat44`.\n\n`a` is the `mat44` to take the second column of.\n\nThis returns a `vec4` representing the second column of `a`.","ref":"Graphmath.Mat44.html#column1/1"},{"type":"function","title":"Graphmath.Mat44.column2/1","doc":"`column2( a )` selects the third column of a `mat44`.\n\n`a` is the `mat44` to take the third column of.\n\nThis returns a `vec4` representing the third column of `a`.","ref":"Graphmath.Mat44.html#column2/1"},{"type":"function","title":"Graphmath.Mat44.column3/1","doc":"`column3( a )` selects the fourth column of a `mat44`.\n\n`a` is the `mat44` to take the fourth column of.\n\nThis returns a `vec4` representing the fourth column of `a`.","ref":"Graphmath.Mat44.html#column3/1"},{"type":"function","title":"Graphmath.Mat44.diag/1","doc":"`diag( a )` selects the diagonal of a `mat44`.\n\n`a` is the `mat44` to take the diagonal of.\n\nThis returns a `vec4` representing the diagonal of `a`.","ref":"Graphmath.Mat44.html#diag/1"},{"type":"function","title":"Graphmath.Mat44.identity/0","doc":"`identity()` creates an identity `mat44`.\n\nThis returns an identity `mat44`.","ref":"Graphmath.Mat44.html#identity/0"},{"type":"function","title":"Graphmath.Mat44.inverse/1","doc":"`inverse(a)` calculates the inverse matrix\n\n`a` is a `mat44` to be inverted\n\nReturs a `mat44` representing `a` -1 \n\nRaises an error when you try to calculate inverse of a matrix whose determinant is `zero`","ref":"Graphmath.Mat44.html#inverse/1"},{"type":"function","title":"Graphmath.Mat44.make_rotate_x/1","doc":"`make_rotate_x( theta )` creates a `mat44` that rotates a `vec3` by `theta` radians about the +X axis.\n\n`theta` is the float of the number of radians of rotation the matrix will provide.\n\nThis returns a `mat44` which rotates by `theta` radians about the +X axis.","ref":"Graphmath.Mat44.html#make_rotate_x/1"},{"type":"function","title":"Graphmath.Mat44.make_rotate_y/1","doc":"`make_rotate_y( theta )` creates a `mat44` that rotates a `vec3` by `theta` radians about the +Y axis.\n\n`theta` is the float of the number of radians of rotation the matrix will provide.\n\nThis returns a `mat44` which rotates by `theta` radians about the +Y axis.","ref":"Graphmath.Mat44.html#make_rotate_y/1"},{"type":"function","title":"Graphmath.Mat44.make_rotate_z/1","doc":"`make_rotate_Z( theta )` creates a `mat44` that rotates a `vec3` by `theta` radians about the +Z axis.\n\n`theta` is the float of the number of radians of rotation the matrix will provide.\n\nThis returns a `mat44` which rotates by `theta` radians about the +Z axis.","ref":"Graphmath.Mat44.html#make_rotate_z/1"},{"type":"function","title":"Graphmath.Mat44.make_scale/1","doc":"`make_scale( k )` creates a `mat44` that uniformly scales.\n\n`k` is the float value to scale by.\n\nThis returns a `mat44` whose diagonal is all `k`s.","ref":"Graphmath.Mat44.html#make_scale/1"},{"type":"function","title":"Graphmath.Mat44.make_scale/4","doc":"`make_scale( sx, sy, sz, sw )` creates a `mat44` that scales each axis independently.\n\n`sx` is a float for scaling the x-axis.\n\n`sy` is a float for scaling the y-axis.\n\n`sz` is a float for scaling the z-axis.\n\n`sw` is a float for scaling the w-axis.\n\nThis returns a `mat44` whose diagonal is `{ sx, sy, sz, sw }`.\n\nNote that, when used with `vec3`s via the *transform* methods, `sw` will have no effect.","ref":"Graphmath.Mat44.html#make_scale/4"},{"type":"function","title":"Graphmath.Mat44.make_translate/3","doc":"`make_translate( tx, ty, tz )` creates a mat44 that translates a point by tx, ty, and tz.\n\n`make_translate( tx, ty, tz )` creates a mat44 that translates a vec3 by (tx, ty, tz).\n\n`tx` is a float for translating along the x-axis.\n\n`ty` is a float for translating along the y-axis.\n\n`tz` is a float for translating along the z-axis.\n\nThis returns a `mat44` which translates by a `vec3` `{ tx, ty, tz }`.","ref":"Graphmath.Mat44.html#make_translate/3"},{"type":"function","title":"Graphmath.Mat44.multiply/2","doc":"`multiply( a, b )` multiply two matrices a and b together.\n\n`a` is the `mat44` multiplicand.\n\n`b` is the `mat44` multiplier.\n\nThis returns the `mat44` product of the `a` and `b`.","ref":"Graphmath.Mat44.html#multiply/2"},{"type":"function","title":"Graphmath.Mat44.multiply_transpose/2","doc":"`multiply_transpose( a, b )` multiply two matrices a and b T  together.\n\n`a` is the `mat44` multiplicand.\n\n`b` is the `mat44` multiplier.\n\nThis returns the `mat44` product of the `a` and `b` T .","ref":"Graphmath.Mat44.html#multiply_transpose/2"},{"type":"function","title":"Graphmath.Mat44.round/2","doc":"`round( a, sigfigs )` rounds every element of a `mat44` to some number of decimal places.\n\n`a` is the `mat44` to round.\n\n`sigfigs` is an integer on [0,15] of the number of decimal places to round to.\n\nThis returns a `mat44` which is the result of rounding `a`.","ref":"Graphmath.Mat44.html#round/2"},{"type":"function","title":"Graphmath.Mat44.row0/1","doc":"`row0( a )` selects the first row of a `mat44`.\n\n`a` is the `mat44` to take the first row of.\n\nThis returns a `vec4` representing the first row of `a`.","ref":"Graphmath.Mat44.html#row0/1"},{"type":"function","title":"Graphmath.Mat44.row1/1","doc":"`row1( a )` selects the second row of a `mat44`.\n\n`a` is the `mat44` to take the second row of.\n\nThis returns a `vec4` representing the second row of `a`.","ref":"Graphmath.Mat44.html#row1/1"},{"type":"function","title":"Graphmath.Mat44.row2/1","doc":"`row2( a )` selects the third row of a `mat44`.\n\n`a` is the `mat44` to take the third row of.\n\nThis returns a `vec4` representing the third row of `a`.","ref":"Graphmath.Mat44.html#row2/1"},{"type":"function","title":"Graphmath.Mat44.row3/1","doc":"`row3( a )` selects the fourth row of a `mat44`.\n\n`a` is the `mat44` to take the fourth row of.\n\nThis returns a `vec4` representing the fourth row of `a`.","ref":"Graphmath.Mat44.html#row3/1"},{"type":"function","title":"Graphmath.Mat44.scale/2","doc":"`scale( a, k )` scales every element in a `mat44` by a coefficient k.\n\n`a` is the `mat44` to scale.\n\n`k` is the float to scale by.\n\nThis returns a `mat44` `a` scaled element-wise by `k`.","ref":"Graphmath.Mat44.html#scale/2"},{"type":"function","title":"Graphmath.Mat44.subtract/2","doc":"`subtract(a,b)` subtracts one `mat44` from another `mat44`.\n\n`a` is the minuend.\n\n`b` is the subtraherd.\n\nThis returns a `mat44` formed by the element-wise subtraction of `b` from `a`.","ref":"Graphmath.Mat44.html#subtract/2"},{"type":"function","title":"Graphmath.Mat44.transform_point/2","doc":"`transform_point( a, v )` transforms a `vec3` point by a `mat44`.\n\n`a` is a `mat44` used to transform the point.\n\n`v` is a `vec3` to be transformed.\n\nThis returns a `vec3` representing the application of `a` to `v`.\n\nThe point `a` is internally treated as having a fourth coordinate equal to 1.0.\n\nNote that transforming a point will work for all transforms.","ref":"Graphmath.Mat44.html#transform_point/2"},{"type":"function","title":"Graphmath.Mat44.transform_vector/2","doc":"`transform_vector( a, v )` transforms a `vec3` vector by a `mat44`.\n\n`a` is a `mat44` used to transform the point.\n\n`v` is a `vec3` to be transformed.\n\nThis returns a `vec3` representing the application of `a` to `v`.\n\nThe point `a` is internally treated as having a fourth coordinate equal to 0.0.\n\nNote that transforming a vector will work for only rotations, scales, and shears.","ref":"Graphmath.Mat44.html#transform_vector/2"},{"type":"function","title":"Graphmath.Mat44.zero/0","doc":"`zero()` creates a zeroed `mat44`.\n\nThis returns a zeroed `mat44`.","ref":"Graphmath.Mat44.html#zero/0"},{"type":"type","title":"Graphmath.Mat44.mat44/0","doc":"","ref":"Graphmath.Mat44.html#t:mat44/0"},{"type":"type","title":"Graphmath.Mat44.vec3/0","doc":"","ref":"Graphmath.Mat44.html#t:vec3/0"},{"type":"type","title":"Graphmath.Mat44.vec4/0","doc":"","ref":"Graphmath.Mat44.html#t:vec4/0"},{"type":"module","title":"Graphmath.Quatern","doc":"This is the 3D mathematics.\n\nThis submodule handles Quaternion using tuples of floats.\n\nQuaternions represent an angle of theta around a unit axis vector {nx, ny, nz} as `{ cos(theta/2), nx * sin(theta/2), ny * sin(theta/2), nz * sin(theta/2) }`.","ref":"Graphmath.Quatern.html"},{"type":"function","title":"Graphmath.Quatern.add/2","doc":"`add(lhs, rhs)` add two quaternions.\n\n`lhs` is the first `quatern`\n\n`rhs` is the second `quatern`\n\nIt returns a `quatern` of the form { lhs w  + rhs w , lhs x  + rhs x , lhs y  + rhs y , lhs z  + rhs z  }.","ref":"Graphmath.Quatern.html#add/2"},{"type":"function","title":"Graphmath.Quatern.conjugate/1","doc":"`conjugate(quat)` returns the conjugate of a quaternion.\n\n`quat` is the quaternion to get the conjugate of.\n\nIt returns a `quatern` representing the inverse of the unit quatern.\n\nNote that the conjugate of a unit quaternion is its inverse.","ref":"Graphmath.Quatern.html#conjugate/1"},{"type":"function","title":"Graphmath.Quatern.create/4","doc":"`create(w,x,y,z)` creates a `quatern` of value (w,x,y,z).\n\n`w` is the rotation around the axis in radians.\n\n`x` is the first element of the `vec3` representing the axis to be created.\n\n`y` is the second element of the `vec3` representing the axis to be created.\n\n`z` is the third element of the `vec3` representing the axis to be created.\n\nIt returns a `quatern` of the form `{w,x,y,z}`.","ref":"Graphmath.Quatern.html#create/4"},{"type":"function","title":"Graphmath.Quatern.dot/2","doc":"`dot(lhs, rhs)` returns a `float` resultant of the dot product bectween two quaterns.\n\n`lhs` is a quatern\n\n`rhs` is a quatern\n\nIt returns a `float` representing the dot product.","ref":"Graphmath.Quatern.html#dot/2"},{"type":"function","title":"Graphmath.Quatern.equal/2","doc":"`equal(a,b)` checks to see if two orientation quaternions a and b are equivalent.\n\n`a` is the first quaternion.\n\n`b` is the second quaternion.\n\nIt returns true if the quaternions represent the same orientation.\n\n**This function expects normalized quaternions.**\n\nNote that orientation quaternions exist where a == -b...that is, where the axes are equivalent but the angle is opposite in sign.","ref":"Graphmath.Quatern.html#equal/2"},{"type":"function","title":"Graphmath.Quatern.equal/3","doc":"`equal(a,b,eps)` checks to see if two orientation quaternions a and b are equivalent up to some epsilon\n\n`a` is the first quaternion.\n\n`b` is the second quaternion.\n\n`eps` is the epsilon, on the interval [0,1].\n\nIt returns true if the quaternions represent the same orientation.\n\n**This function expects normalized quaternions.**\n\nNote that orientation quaternions exist where a == -b...that is, where the axes are equivalent but the angle is opposite in sign.","ref":"Graphmath.Quatern.html#equal/3"},{"type":"function","title":"Graphmath.Quatern.equal_elements/2","doc":"`equal_elements(a,b)` checks to see if two quaternions a and b are element-wise equal.\n\n`a` is the first quaternion.\n\n`b` is the second quaternion.\n\nIt returns true if the quaternions have the same elements, false otherwise.\n\n**This function does not require normalized quaternions.**\n\nNote that orientation quaternions exist where a == -b...that is, where the axes are equivalent but the angle is opposite in sign.\n\nIn such cases, prefer the `equal/2` function.","ref":"Graphmath.Quatern.html#equal_elements/2"},{"type":"function","title":"Graphmath.Quatern.equal_elements/3","doc":"`equal_elements(a,b, eps)` checks to see if two quaternions a and b are element-wise equal to some epsilon\n\n`a` is the first quaternion.\n\n`b` is the second quaternion.\n\n`eps` is the float of the epsilon for comparison.\n\nIt returns true if the quaternions have the same element-wise values up to and including some epsilon.\n\n**This function does not require normalized quaternions.**\n\nNote that orientation quaternions exist where a == -b...that is, where the axes are equivalent but the angle is opposite in sign.\n\nIn such cases, prefer the `equal/3` function.","ref":"Graphmath.Quatern.html#equal_elements/3"},{"type":"function","title":"Graphmath.Quatern.from_axis_angle/2","doc":"`create(w, vec)` creates a `quatern` from an angle and an axis.\n\n`w` is the angle in radians.\n\n`vec` is the axis `vec3` of the form {x,y,z}.\n\nIt returns a `quatern` of the form `{w,x,y,z}`.","ref":"Graphmath.Quatern.html#from_axis_angle/2"},{"type":"function","title":"Graphmath.Quatern.from_list/1","doc":"`create(quatern)` creates a `quatern` from a list of 4 or more floats.\n\n`quatern` is a list of 4 or more floats.\n\nIt returns a `quatern` of the form `{w,x,y,z}`, where `w`, `x`, `y`, and `z` are the first four elements in `quatern`.","ref":"Graphmath.Quatern.html#from_list/1"},{"type":"function","title":"Graphmath.Quatern.from_rotation_matrix/1","doc":"`from_rotation_matrix(mat)` creates a `quatern` from a rotation matrix.\n\n`mat` is the matrix\n\nIt returns a `quatern` of the form `{w,x,y,z}`.","ref":"Graphmath.Quatern.html#from_rotation_matrix/1"},{"type":"function","title":"Graphmath.Quatern.get_pitch/1","doc":"`pitch(quat)` Calculate the local pitch element of a quaternion.\n\n`quat` is the quatern\n\nIt returns a `float` representing the pitch of the quaternion in Radians.","ref":"Graphmath.Quatern.html#get_pitch/1"},{"type":"function","title":"Graphmath.Quatern.get_roll/1","doc":"`roll(quat)` Calculate the local roll element of a quaternion.\n\n`quat` is the quatern\n\nIt returns a `float` representing the roll of the quaternion in Radians.","ref":"Graphmath.Quatern.html#get_roll/1"},{"type":"function","title":"Graphmath.Quatern.get_yaw/1","doc":"`yaw(quat)` Calculate the local yaw element of a quaternion.\n\n`quat` is the quatern\n\nIt returns a `float` representing the yaw of the quaternion in Radians.","ref":"Graphmath.Quatern.html#get_yaw/1"},{"type":"function","title":"Graphmath.Quatern.identity/0","doc":"`identity()` creates the identity `quatern`.\n\nIt takes no arguments.\n\nIt returns a `quatern` of the form `{1.0, 0.0, 0.0, 0.0}`.","ref":"Graphmath.Quatern.html#identity/0"},{"type":"function","title":"Graphmath.Quatern.integrate/3","doc":"`integrate(q, omega, dt)` integrates the angular velocty omega over a timestep dt with intial orientation q.\n\n`q` is an orientation quaternion to use as the initial orientation.\n\n`omega` is a `vec3` whose direction is the axis of rotation and whose magnitude is the velocity about that axis.\n\n`dt` is the timestep over which to apply the angular velocity.\n\nIt returns an orientation `quatern`.","ref":"Graphmath.Quatern.html#integrate/3"},{"type":"function","title":"Graphmath.Quatern.inverse/1","doc":"`inverse(quat)` returns the inverse of a quaternion.\n\n`quat` is the quaternion\n\nIt returns a `quatern` representing the inverse of the parameter quaternion.\n\nIf the `quat` is less than or equal to zero, the quaternion returned is a zero quaternion.","ref":"Graphmath.Quatern.html#inverse/1"},{"type":"function","title":"Graphmath.Quatern.multiply/2","doc":"`multiply(lhs, rhs)` multiply two quaternions.\n\n `lhs` is the first `quatern`\n\n `rhs` is the second `quatern`\n\n It returns a `quatern` resultant of the multiplication\n NOTE: Multiplication is not generally commutative, so in most cases p*q != q*p.","ref":"Graphmath.Quatern.html#multiply/2"},{"type":"function","title":"Graphmath.Quatern.norm/1","doc":"`norm(quat)` Returns the L2 norm of a quaternion.\n\n`quat` is a `quatern` to find the norm of.\n\nIt returns a `float` representing the L2 norm.","ref":"Graphmath.Quatern.html#norm/1"},{"type":"function","title":"Graphmath.Quatern.normalize/1","doc":"`normalize(q)` returns a normalized verison of a quaternion.\n\n`q` is the `quatern` to be normalized.\n\nThis returns a `quatern` of unit length in the same direction as `q`.\n\nIf the magnitude of the quaternion is 0, it will return the zero quaternion.","ref":"Graphmath.Quatern.html#normalize/1"},{"type":"function","title":"Graphmath.Quatern.normalize_strict/1","doc":"`normalize_strict(q)` returns a normalized verison of a quaternion.\n\n`q` is the `quatern` to be normalized.\n\nThis returns a `quatern` of unit length in the same direction as `q`.\n\nIf the magnitude of the quaternion is 0, it will explode.","ref":"Graphmath.Quatern.html#normalize_strict/1"},{"type":"function","title":"Graphmath.Quatern.random/0","doc":"Genrate a random quatenrion (rotation on SO3), using the algorithm given [here](http://planning.cs.uiuc.edu/node198.html).\n\nIt returns an orientation `quatern`.","ref":"Graphmath.Quatern.html#random/0"},{"type":"function","title":"Graphmath.Quatern.scale/2","doc":"`scale(quat, scalar)` multiply a `quatern` for a scalar.\n\n `quat` is the `quatern`\n\n `scalar` is the scalar\n\n It returns a `quatern` of the form\n    { a w  * scalar, a x  * scalar, a y  * scalar, a z  * scalar}.","ref":"Graphmath.Quatern.html#scale/2"},{"type":"function","title":"Graphmath.Quatern.slerp/3","doc":"`slerp(lhs, rhs, t)` Performs Spherical linear interpolation between two quaternions, and returns the result.\n\n`lhs` is the first `quatern`\n\n`rhs` is the second `quatern`\n\n`t` is the interpolation parameter that will interpolate to `lhs` when `t = 0` and to `rhs` when `t = 1`.\n\nIt returns a `quatern` representing the normalized interpolation point.\n\nNote: `slerp` has the proprieties of performing the interpolation at constant velocity However, it's NOT commutative, which means\n`slerp( A, B, 0.75 ) != slerp( B, A, 0.25 )`\ntherefore be careful if your code relies in the order of the operands.\nThis is specially important in IK animation.","ref":"Graphmath.Quatern.html#slerp/3"},{"type":"function","title":"Graphmath.Quatern.subtract/2","doc":"`subtract(lhs, rhs)` subtract two quaternions.\n\n `lhs` is the first `quatern`\n\n `rhs` is the second `quatern`\n\n It returns a `quatern` of the form { lhs w  - rhs w , lhs x  - rhs x , lhs y  - rhs y , lhs z  - rhs z  }.","ref":"Graphmath.Quatern.html#subtract/2"},{"type":"function","title":"Graphmath.Quatern.to_rotation_matrix_33/1","doc":"`to_rotation_matrix_33(quat)` creates a `mat33` from a quatern.\n\n`quat` is the quatern\n\nIt returns a `mat33` representing a rotation.","ref":"Graphmath.Quatern.html#to_rotation_matrix_33/1"},{"type":"function","title":"Graphmath.Quatern.to_rotation_matrix_44/1","doc":"`to_rotation_matrix_44(quat)` creates a `mat44` from a quatern.\n\n`quat` is the quatern\n\nIt returns a `mat44` representing a rotation.","ref":"Graphmath.Quatern.html#to_rotation_matrix_44/1"},{"type":"function","title":"Graphmath.Quatern.transform_vector/2","doc":"`transform_vector(q,v)` transforms a vector v by an orientation quaternion q.\n\n`q` is an orientation quaternion.\n\n`v` is a Vec3 to transform--it need not be normalized.\n\nIt returns a `Vec3` of `v` having undergone the rotation represented by `q`.","ref":"Graphmath.Quatern.html#transform_vector/2"},{"type":"function","title":"Graphmath.Quatern.zero/0","doc":"`zero()` creates a zero `quatern`.\n\nIt takes no arguments.\n\nIt returns a `quatern` of the form `{0.0, 0.0, 0.0, 0.0}`.\n\nNote that the zero quaternion is almost definetely not something you ever use.","ref":"Graphmath.Quatern.html#zero/0"},{"type":"type","title":"Graphmath.Quatern.mat33/0","doc":"","ref":"Graphmath.Quatern.html#t:mat33/0"},{"type":"type","title":"Graphmath.Quatern.mat44/0","doc":"","ref":"Graphmath.Quatern.html#t:mat44/0"},{"type":"type","title":"Graphmath.Quatern.quatern/0","doc":"","ref":"Graphmath.Quatern.html#t:quatern/0"},{"type":"type","title":"Graphmath.Quatern.vec3/0","doc":"","ref":"Graphmath.Quatern.html#t:vec3/0"},{"type":"module","title":"Graphmath.Vec2","doc":"This is the 2D mathematics.\n\nThis submodule handles vectors stored as tuples of floats ex: `{1.0, 2.0}`.","ref":"Graphmath.Vec2.html"},{"type":"function","title":"Graphmath.Vec2.add/2","doc":"`add( a, b)` adds a vec2 (a) to a vec2 (b).\n\nIt returns a tuple of the form { ax + bx, ay + by }.\n\n`add( a, b )` adds two `vec2`s.\n\n`a` is the first `vec2`.\n\n`b` is the second `vec2`.\n\nIt returns a `vec2` of the form { a x  + b x , a y  + b y  }.","ref":"Graphmath.Vec2.html#add/2"},{"type":"function","title":"Graphmath.Vec2.chebyshev_distance/2","doc":"`chebyshev_distance(a,b)` returns the [Chebyshev distance](https://en.wikipedia.org/wiki/Chebyshev_distance) between two points `a` and `b`.","ref":"Graphmath.Vec2.html#chebyshev_distance/2"},{"type":"function","title":"Graphmath.Vec2.create/0","doc":"`create()` creates a zero vec2.\n\nIt will return a tuple of the form {0.0,0.0}.\n`create()` creates a zeroed `vec2`.\n\nIt takes no arguments.\n\nIt returns a `vec2` of the form `{ 0.0, 0.0 }`.","ref":"Graphmath.Vec2.html#create/0"},{"type":"function","title":"Graphmath.Vec2.create/1","doc":"`create(vec)` creates a `vec2` from a list of 2 or more floats.\n\n`vec` is a list of 2 or more floats.\n\nIt returns a `vec2` of the form `{x,y}`, where `x` and `y` are the first three elements in `vec`.","ref":"Graphmath.Vec2.html#create/1"},{"type":"function","title":"Graphmath.Vec2.create/2","doc":"`create(x,y)` creates a `vec2` of value (x,y).\n\n`x` is the first element of the `vec3` to be created.\n\n`y` is the second element of the `vec3` to be created.\n\nIt returns a `vec2` of the form `{x,y}`.","ref":"Graphmath.Vec2.html#create/2"},{"type":"function","title":"Graphmath.Vec2.dot/2","doc":"`dot( a, b )` finds the dot (inner) product of one `vec2` with another `vec2`.\n\n`a` is the first `vec2`.\n\n`b` is the second `vec2`.\n\nIt returns a float of the value (a x b x  + a y b y  ).","ref":"Graphmath.Vec2.html#dot/2"},{"type":"function","title":"Graphmath.Vec2.equal/2","doc":"`equal(a, b)` checks to see if two vec2s a and b are equivalent.\n\n`a` is the `vec2`.\n\n`b` is the `vec2`.\n\nIt returns true if the vectors have equal elements.\n\nNote that due to precision issues, you may want to use `equal/3` instead.","ref":"Graphmath.Vec2.html#equal/2"},{"type":"function","title":"Graphmath.Vec2.equal/3","doc":"`equal(a, b, eps)` checks to see if two vec2s a and b are equivalent within some tolerance.\n\n`a` is the `vec2`.\n\n`b` is the `vec2`.\n\n`eps` is the tolerance, a float.\n\nIt returns true if the vectors have equal elements within some tolerance.","ref":"Graphmath.Vec2.html#equal/3"},{"type":"function","title":"Graphmath.Vec2.length/1","doc":"`length(a)` finds the length (Eucldiean or L2 norm) of a `vec2`.\n\n`a` is the `vec2` to find the length of.\n\nIt returns a float of the value (sqrt( a x  2  + a y  2 )).","ref":"Graphmath.Vec2.html#length/1"},{"type":"function","title":"Graphmath.Vec2.length_manhattan/1","doc":"`length_manhattan(a)` finds the Manhattan (L1 norm) length of a `vec2`.\n\n`a` is the `vec2` to find the Manhattan length of.\n\nIt returns a float of the value (a x  + a y ).\n\nThe Manhattan length is the sum of the components.","ref":"Graphmath.Vec2.html#length_manhattan/1"},{"type":"function","title":"Graphmath.Vec2.length_squared/1","doc":"`length_squared(a)` finds the square of the length of a vec2 (a).\n\nIn many cases, this is sufficient for comparisions and avaoids a sqrt.\n\nIt returns a float of the value (ax*ax + ay*ay).\n`length_squared(a)` finds the square of the length of a `vec2`.\n\n`a` is the `vec2` to find the length squared of.\n\nIt returns a float of the value a x  2  + a y  2 .\n\nIn many cases, this is sufficient for comparisons and avoids a square root.","ref":"Graphmath.Vec2.html#length_squared/1"},{"type":"function","title":"Graphmath.Vec2.lerp/3","doc":"`lerp(a,b,t)` is used to linearly interpolate between two given vectors a and b along an interpolant t.\n\nThe interpolant `t`  is on the domain [0,1]. Behavior outside of that is undefined.\n`lerp(a,b,t)` linearly interpolates between one `vec2` and another `vec2` along an interpolant.\n\n`a` is the starting `vec2`.\n\n`b` is the ending `vec2`.\n\n`t` is the interpolant float, on the domain [0,1].\n\nIt returns a `vec2` of the form (1-t)**a** - (t)**b**.\n\nThe interpolant `t` is on the domain [0,1]. Behavior outside of that is undefined.","ref":"Graphmath.Vec2.html#lerp/3"},{"type":"function","title":"Graphmath.Vec2.minkowski_distance/3","doc":"`minkowski_distance(a,b,order)` returns the [Minkowski distance](https://en.wikipedia.org/wiki/Minkowski_distance) between two points `a` and `b` of order `order`.\n\nOrder 1 is equivalent to manhattan distance, 2 to Euclidean distance, otherwise all bets are off.","ref":"Graphmath.Vec2.html#minkowski_distance/3"},{"type":"function","title":"Graphmath.Vec2.multiply/2","doc":"`multiply( a, b)` mulitplies element-wise a vec2 (a) by a vec2 (b).\n\nIt returns a tuple of the form `{ ax*bx, ay*by }`.\n\n`multiply( a, b )` multiplies element-wise a `vec2` by another `vec2`.\n\n`a` is the `vec2` multiplicand.\n\n`b` is the `vec2` multiplier.\n\nIt returns a `vec2` of the form { a x b x , a y b y  }.","ref":"Graphmath.Vec2.html#multiply/2"},{"type":"function","title":"Graphmath.Vec2.near/3","doc":"`near(a,b, distance)` checks whether two `vec2`s are within a certain distance of each other.\n\n`a` is the first `vec2`.\n\n`b` is the second `vec2`.\n\n`distance` is the distance between them as a float.","ref":"Graphmath.Vec2.html#near/3"},{"type":"function","title":"Graphmath.Vec2.negate/1","doc":"`negate(v)` creates a vector whose elements are opposite in sign to `v`.","ref":"Graphmath.Vec2.html#negate/1"},{"type":"function","title":"Graphmath.Vec2.normalize/1","doc":"`normalize(a)` finds the unit vector with the same direction as a `vec2`.\n\n`a` is the `vec2` to be normalized.\n\nIt returns a `vec2` of the form `{normx, normy}`.\n\nThis is done by dividing each component by the vector's magnitude.","ref":"Graphmath.Vec2.html#normalize/1"},{"type":"function","title":"Graphmath.Vec2.p_norm/2","doc":"`p_norm(v,order)` returns the [P-norm](https://en.wikipedia.org/wiki/Lp_space#The_p-norm_in_finite_dimensions) of vector `v` of order `order`.\n\n`order` needs to be greater than or equal to 1 to define a [metric space](https://en.wikipedia.org/wiki/Metric_space).\n\n`order` 1 is equivalent to manhattan distance, 2 to Euclidean distance, otherwise all bets are off.","ref":"Graphmath.Vec2.html#p_norm/2"},{"type":"function","title":"Graphmath.Vec2.perp/1","doc":"`perp(a)` creates a vector perpendicular to another vector `a`.\n\n`a` is the `vec2` to be perpindicular to.\n\nThis returns a `vec2` perpindicular to `a`, to the right of the original `a`.","ref":"Graphmath.Vec2.html#perp/1"},{"type":"function","title":"Graphmath.Vec2.perp_prod/2","doc":"`perp_prod( a, b )` finds the perpindicular product of one `vec2` with another `vec2`.\n\n`a` is the first `vec2`.\n\n`b` is the second `vec2`.\n\nThe perpindicular product is the magnitude of the cross-product between the two vectors.\n\nIt returns a float of the value (a x b y  - b x a y ).","ref":"Graphmath.Vec2.html#perp_prod/2"},{"type":"function","title":"Graphmath.Vec2.project/2","doc":"`project(a,b)` projects one `vec2` onto another `vec2`.\n\n`a` is the first `vec2`.\n\n`b` is the second `vec2`.\n\nThis returns a `vec2` representing the image of `a` in the direction of `b`.","ref":"Graphmath.Vec2.html#project/2"},{"type":"function","title":"Graphmath.Vec2.random_box/0","doc":"`random_box()` generates a point on or inside the unit box [0,1]x[0,1].","ref":"Graphmath.Vec2.html#random_box/0"},{"type":"function","title":"Graphmath.Vec2.random_circle/0","doc":"`random_circle()` generates a point on the unit circle.\n\nIt returns a vec2 with distance 1 from the origin.","ref":"Graphmath.Vec2.html#random_circle/0"},{"type":"function","title":"Graphmath.Vec2.random_disc/0","doc":"`random_disc()` generates a point on or inside the unit circle using the method [here](http://mathworld.wolfram.com/DiskPointPicking.html).\n\nIt returns a vec2 with distance 1 from the origin.","ref":"Graphmath.Vec2.html#random_disc/0"},{"type":"function","title":"Graphmath.Vec2.rotate/2","doc":"`rotate(a,theta)` rotates a `vec2` CCW about the +Z axis.\n\n`a` is the `vec2` to rotate.\n\n`theta` is the number of radians to rotate by as a float.\n\nThis returns a `vec2`.","ref":"Graphmath.Vec2.html#rotate/2"},{"type":"function","title":"Graphmath.Vec2.scale/2","doc":"`scale( a, scale )` uniformly scales a `vec2`.\n\n`a` is the `vec2` to be scaled.\n\n`scale` is the float to scale each element of `a` by.\n\nIt returns a tuple of the form { a x scale, a y scale }.","ref":"Graphmath.Vec2.html#scale/2"},{"type":"function","title":"Graphmath.Vec2.subtract/2","doc":"`subtract(a, b )` subtracts one `vec2` from another `vec2`.\n\n`a` is the `vec2` minuend.\n\n`b` is the `vec2` subtrahend.\n\nIt returns a `vec2` of the form { a x  - b x , a y  - b y  }.\n\n(the terminology was found [here](http://mathforum.org/library/drmath/view/58801.html)).","ref":"Graphmath.Vec2.html#subtract/2"},{"type":"function","title":"Graphmath.Vec2.weighted_sum/4","doc":"`weighted_sum(a, v1, b, v2)` returns the sum of vectors `v1` and `v2` having been scaled by `a` and `b`, respectively.","ref":"Graphmath.Vec2.html#weighted_sum/4"},{"type":"type","title":"Graphmath.Vec2.vec2/0","doc":"","ref":"Graphmath.Vec2.html#t:vec2/0"},{"type":"module","title":"Graphmath.Vec3","doc":"This is the 3D mathematics.\n\nThis submodule handles 3D vectors using tuples of floats ex: `{1.0, 2.0, 3.0}`.","ref":"Graphmath.Vec3.html"},{"type":"function","title":"Graphmath.Vec3.add/2","doc":"`add( a, b)` adds two `vec3`s.\n\n`a` is the first `vec3`.\n\n`b` is the second `vec3`.\n\nIt returns a `vec3` of the form { a x  + b x , a y  + b y , a z  + b z  }.","ref":"Graphmath.Vec3.html#add/2"},{"type":"function","title":"Graphmath.Vec3.chebyshev_distance/2","doc":"`chebyshev_distance(a,b)` returns the [Chebyshev distance](https://en.wikipedia.org/wiki/Chebyshev_distance) between two points `a` and `b`.","ref":"Graphmath.Vec3.html#chebyshev_distance/2"},{"type":"function","title":"Graphmath.Vec3.create/0","doc":"`create()` creates a zeroed `vec3`.\n\nIt takes no arguments.\n\nIt returns a `vec3` of the form `{ 0.0, 0.0, 0.0 }`.","ref":"Graphmath.Vec3.html#create/0"},{"type":"function","title":"Graphmath.Vec3.create/1","doc":"`create(vec)` creates a `vec3` from a list of 3 or more floats.\n\n`vec` is a list of 3 or more floats.\n\nIt returns a `vec3` of the form `{x,y,z}`, where `x`, `y`, and `z` are the first three elements in `vec`.","ref":"Graphmath.Vec3.html#create/1"},{"type":"function","title":"Graphmath.Vec3.create/3","doc":"`create(x,y,z)` creates a `vec3` of value (x,y,z).\n\n`x` is the first element of the `vec3` to be created.\n\n`y` is the second element of the `vec3` to be created.\n\n`z` is the third element of the `vec3` to be created.\n\nIt returns a `vec3` of the form `{x,y,z}`.","ref":"Graphmath.Vec3.html#create/3"},{"type":"function","title":"Graphmath.Vec3.cross/2","doc":"`cross( a, b)` finds the cross productof one `vec3` with another `vec3`.\n\n`a` is the first `vec3`.\n\n`b` is the second `vec3`.\n\nIt returns a float of the value ( a y b z  - a z b y , a z b x  - a x b z , a x b y  - a y b x ).\n\nThe cross product of two vectors is a vector perpendicular to the two source vectors.\nIts magnitude will be the area of the parallelogram made by the two souce vectors.","ref":"Graphmath.Vec3.html#cross/2"},{"type":"function","title":"Graphmath.Vec3.dot/2","doc":"`dot( a, b)` finds the dot (inner) product of one `vec3` with another `vec3`.\n\n`a` is the first `vec3`.\n\n`b` is the second `vec3`.\n\nIt returns a float of the value (a x b x  + a y b y  + a z b z ).","ref":"Graphmath.Vec3.html#dot/2"},{"type":"function","title":"Graphmath.Vec3.equal/2","doc":"`equal(a, b)` checks to see if two vec3s a and b are equivalent.\n\n`a` is the `vec3`.\n\n`b` is the `vec3`.\n\nIt returns true if the vectors have equal elements.\n\nNote that due to precision issues, you may want to use `equal/3` instead.","ref":"Graphmath.Vec3.html#equal/2"},{"type":"function","title":"Graphmath.Vec3.equal/3","doc":"`equal(a, b, eps)` checks to see if two vec3s a and b are equivalent within some tolerance.\n\n`a` is the `vec3`.\n\n`b` is the `vec3`.\n\n`eps` is the tolerance, a float.\n\nIt returns true if the vectors have equal elements within some tolerance.","ref":"Graphmath.Vec3.html#equal/3"},{"type":"function","title":"Graphmath.Vec3.length/1","doc":"`length(a)` finds the length (Eucldiean or L2 norm) of a `vec3`.\n\n`a` is the `vec3` to find the length of.\n\nIt returns a float of the value (sqrt( a x  2  + a y  2  + a z  2 )).","ref":"Graphmath.Vec3.html#length/1"},{"type":"function","title":"Graphmath.Vec3.length_manhattan/1","doc":"`length_manhattan(a)` finds the Manhattan (L1 norm) length of a `vec3`.\n\n`a` is the `vec3` to find the Manhattan length of.\n\nIt returns a float of the value (a x  + a y  + a z ).\n\nThe Manhattan length is the sum of the components.","ref":"Graphmath.Vec3.html#length_manhattan/1"},{"type":"function","title":"Graphmath.Vec3.length_squared/1","doc":"`length_squared(a)` finds the square of the length of a `vec3`.\n\n`a` is the `vec3` to find the length squared of.\n\nIt returns a float of the value a x  2  + a y  2  + a z  2 .\n\nIn many cases, this is sufficient for comparisons and avoids a square root.","ref":"Graphmath.Vec3.html#length_squared/1"},{"type":"function","title":"Graphmath.Vec3.lerp/3","doc":"`lerp(a,b,t)` linearly interpolates between one `vec3` and another `vec3` along an interpolant.\n\n`a` is the starting `vec3`.\n\n`b` is the ending `vec3`.\n\n`t` is the interpolant float, on the domain [0,1].\n\nIt returns a `vec3` of the form (1-t)**a** - (t)**b**.\n\nThe interpolant `t` is on the domain [0,1]. Behavior outside of that is undefined.","ref":"Graphmath.Vec3.html#lerp/3"},{"type":"function","title":"Graphmath.Vec3.minkowski_distance/3","doc":"`minkowski_distance(a,b,order)` returns the [Minkowski distance](https://en.wikipedia.org/wiki/Minkowski_distance) between two points `a` and `b` of order `order`.\n\n`order` needs to be greater than or equal to 1 to define a [metric space](https://en.wikipedia.org/wiki/Metric_space).\n\n`order` 1 is equivalent to manhattan distance, 2 to Euclidean distance, otherwise all bets are off.","ref":"Graphmath.Vec3.html#minkowski_distance/3"},{"type":"function","title":"Graphmath.Vec3.multiply/2","doc":"`multiply( a, b)` multiplies element-wise a `vec3` by another `vec3`.\n\n`a` is the `vec3` multiplicand.\n\n`b` is the `vec3` multiplier.\n\nIt returns a `vec3` of the form { a x b x , a y b y , a z b z  }.","ref":"Graphmath.Vec3.html#multiply/2"},{"type":"function","title":"Graphmath.Vec3.near/3","doc":"`near(a,b, distance)` checks whether two `vec3`s are within a certain distance of each other.\n\n`a` is the first `vec3`.\n\n`b` is the second `vec3`.\n\n`distance` is the distance between them as a float.","ref":"Graphmath.Vec3.html#near/3"},{"type":"function","title":"Graphmath.Vec3.negate/1","doc":"`negate(v)` creates a vector whose elements are opposite in sign to `v`.","ref":"Graphmath.Vec3.html#negate/1"},{"type":"function","title":"Graphmath.Vec3.normalize/1","doc":"`normalize(a)` finds the unit vector with the same direction as a `vec3`.\n\n`a` is the `vec3` to be normalized.\n\nIt returns a `vec3` of the form `{normx, normy, normz}`.\n\nThis is done by dividing each component by the vector's magnitude.","ref":"Graphmath.Vec3.html#normalize/1"},{"type":"function","title":"Graphmath.Vec3.p_norm/2","doc":"`p_norm(v,order)` returns the [P-norm](https://en.wikipedia.org/wiki/Lp_space#The_p-norm_in_finite_dimensions) of vector `v` of order `order`.\n\n`order` needs to be greater than or equal to 1 to define a [metric space](https://en.wikipedia.org/wiki/Metric_space).\n\n`order` 1 is equivalent to manhattan distance, 2 to Euclidean distance, otherwise all bets are off.","ref":"Graphmath.Vec3.html#p_norm/2"},{"type":"function","title":"Graphmath.Vec3.random_ball/0","doc":"`random_ball()` gives a point at or within unit distance of the origin, using [the last algo here](https://karthikkaranth.me/blog/generating-random-points-in-a-sphere/).\n\nIt returns a vec3 within at most unit distance of the origin.","ref":"Graphmath.Vec3.html#random_ball/0"},{"type":"function","title":"Graphmath.Vec3.random_box/0","doc":"`random_box()` gives a point on or in the unit box [0,1]x[0,1]x[0,1].\n\nIt returns a vec3.","ref":"Graphmath.Vec3.html#random_box/0"},{"type":"function","title":"Graphmath.Vec3.random_sphere/0","doc":"`random_sphere()` gives a point at or within unit distance of the origin, using [this](http://extremelearning.com.au/how-to-generate-uniformly-random-points-on-n-spheres-and-n-balls/) polar method.\nAnother really nice exploration of this is [here](http://mathworld.wolfram.com/SpherePointPicking.html).\n\nIt returns a vec3 within at most unit distance of the origin.","ref":"Graphmath.Vec3.html#random_sphere/0"},{"type":"function","title":"Graphmath.Vec3.rotate/3","doc":"`rotate( v, k, theta)` rotates a vector (v) about a unit vector (k) by theta radians.\n\n`v` is the `vec3` to be rotated.\n\n`k` is the `vec3` axis of rotation. *It must be of unit length*.\n\n`theta` is the angle in radians to rotate as a float.\n\nThis uses the [Formula of Rodriguez](http://en.wikipedia.org/wiki/Rodrigues%27_rotation_formula):\n\n**V** rot  = **V**cos(theta) + (**K** x **V**)sin(theta) + **K**(**K** dot **V**)(1-cos(theta))","ref":"Graphmath.Vec3.html#rotate/3"},{"type":"function","title":"Graphmath.Vec3.scalar_triple/3","doc":"`scalar_triple(a,b,c)` returns the [scalar triple product](https://en.wikipedia.org/wiki/Triple_product#Scalar_triple_product) of three vectors.\n\nWe're using the `a*(b x c)` form.","ref":"Graphmath.Vec3.html#scalar_triple/3"},{"type":"function","title":"Graphmath.Vec3.scale/2","doc":"`scale( a, scale)` uniformly scales a `vec3`.\n\n`a` is the `vec3` to be scaled.\n\n`scale` is the float to scale each element of `a` by.\n\nIt returns a tuple of the form { a x scale, a y scale, a z scale }.","ref":"Graphmath.Vec3.html#scale/2"},{"type":"function","title":"Graphmath.Vec3.subtract/2","doc":"`subtract(a, b)` subtracts one `vec3` from another `vec3`.\n\n`a` is the `vec3` minuend.\n\n`b` is the `vec3` subtrahend.\n\nIt returns a `vec3` of the form { a x  - b x , a y  - b y , a z  - b z  }.\n\n(the terminology was found [here](http://mathforum.org/library/drmath/view/58801.html)).","ref":"Graphmath.Vec3.html#subtract/2"},{"type":"function","title":"Graphmath.Vec3.weighted_sum/4","doc":"`weighted_sum(a, v1, b, v2)` returns the sum of vectors `v1` and `v2` having been scaled by `a` and `b`, respectively.","ref":"Graphmath.Vec3.html#weighted_sum/4"},{"type":"type","title":"Graphmath.Vec3.vec3/0","doc":"","ref":"Graphmath.Vec3.html#t:vec3/0"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}